"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _isomorphicWebcrypto = _interopRequireDefault(require("isomorphic-webcrypto"));

var qs = _interopRequireWildcard(require("qs"));

var _axios = _interopRequireDefault(require("axios"));

var _textEncodingPolyfill = require("text-encoding-polyfill");

var _jsrsasign = require("jsrsasign");

require("hex-lite");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const ENVIRONMENT = {
  SANDBOX: 'sandbox',
  PRODUCTION: 'production'
};

class JunoCardHash {
  constructor(publicToken, environment = 'sandbox') {
    _defineProperty(this, "publicToken", void 0);

    _defineProperty(this, "environment", void 0);

    _defineProperty(this, "axios", void 0);

    this.publicToken = publicToken;
    this.environment = environment;
    this.axios = this._configureAxios(this.environment);
  }

  static getAlgorithm() {
    return {
      name: 'RSA-OAEP',
      hash: {
        name: 'SHA-256'
      }
    };
  }

  async getCardHash(cardData) {
    const publicKey = await this._fetchPublicKey();

    const key = _jsrsasign.KEYUTIL.getKey(publicKey);

    const jwkKey = _jsrsasign.KEYUTIL.getJWKFromKey(key);

    const encriptedPublicKey = await this._importKey(jwkKey);

    const cardBuffer = this._str2ab(JSON.stringify(cardData));

    const encryptedCard = await this._encryptCardData(encriptedPublicKey, cardBuffer);
    const result = await this._fetchCardHash(encryptedCard);

    if (!result.data) {
      throw new Error('Não foi possível gerar o hash do cartão');
    }

    return result.data;
  }

  async _fetchPublicKey() {
    const params = qs.stringify({
      publicToken: this.publicToken
    });
    const ENDPOINT = "/get-public-encryption-key.json?".concat(params);

    try {
      let {
        data
      } = await this.axios.post(ENDPOINT);
      return "-----BEGIN PUBLIC KEY-----\n".concat(data, "\n-----END PUBLIC KEY-----");
    } catch (error) {
      var _error$response;

      throw new Error((error === null || error === void 0 ? void 0 : (_error$response = error.response) === null || _error$response === void 0 ? void 0 : _error$response.message) || 'Erro ao gerar a chave pública na API de pagamentos');
    }
  }

  _fetchCardHash(encryptedCard) {
    const params = qs.stringify({
      publicToken: this.publicToken,
      encryptedData: encryptedCard
    });
    const ENDPOINT = "/get-credit-card-hash.json?".concat(params);
    return this.axios.post(ENDPOINT);
  }

  _importKey(binaryKey) {
    const algorithm = JunoCardHash.getAlgorithm();
    return _isomorphicWebcrypto.default.ensureSecure().then(() => // @ts-ignore
    _isomorphicWebcrypto.default.subtle.importKey('spki', binaryKey, algorithm, false, ['encrypt']));
  }

  _encryptCardData(publicKey, encodedCardData) {
    const algorithm = JunoCardHash.getAlgorithm();
    return new Promise((resolve, reject) => _isomorphicWebcrypto.default.ensureSecure().then(() => // @ts-ignore
    _isomorphicWebcrypto.default.subtle.encrypt(algorithm, publicKey, encodedCardData).then(data => this._encodeAb(data), reject).then(encoded => resolve(encoded))));
  }

  _str2ab(str) {
    var encoder = new _textEncodingPolyfill.TextEncoder();
    var byteArray = encoder.encode(str);
    return byteArray.buffer;
  }

  _encodeAb(arrayBuffer) {
    let base64 = '';
    const encodings = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
    const bytes = new Uint8Array(arrayBuffer);
    const {
      byteLength
    } = bytes;
    const byteRemainder = byteLength % 3;
    const mainLength = byteLength - byteRemainder;
    let a;
    let b;
    let c;
    let d;
    let chunk; // Main loop deals with bytes in chunks of 3

    for (let i = 0; i < mainLength; i += 3) {
      // Combine the three bytes into a single integer
      chunk = bytes[i] << 16 | bytes[i + 1] << 8 | bytes[i + 2]; // Use bitmasks to extract 6-bit segments from the triplet

      a = (chunk & 16515072) >> 18; // 16515072 = (2^6 - 1) << 18

      b = (chunk & 258048) >> 12; // 258048   = (2^6 - 1) << 12

      c = (chunk & 4032) >> 6; // 4032     = (2^6 - 1) << 6

      d = chunk & 63; // 63       = 2^6 - 1
      // Convert the raw binary segments to the appropriate ASCII encoding

      base64 += encodings[a] + encodings[b] + encodings[c] + encodings[d];
    } // Deal with the remaining bytes and padding


    if (byteRemainder === 1) {
      chunk = bytes[mainLength];
      a = (chunk & 252) >> 2; // 252 = (2^6 - 1) << 2
      // Set the 4 least significant bits to zero

      b = (chunk & 3) << 4; // 3   = 2^2 - 1

      base64 += "".concat(encodings[a] + encodings[b], "==");
    } else if (byteRemainder === 2) {
      chunk = bytes[mainLength] << 8 | bytes[mainLength + 1];
      a = (chunk & 64512) >> 10; // 64512 = (2^6 - 1) << 10

      b = (chunk & 1008) >> 4; // 1008  = (2^6 - 1) << 4
      // Set the 2 least significant bits to zero

      c = (chunk & 15) << 2; // 15    = 2^4 - 1

      base64 += "".concat(encodings[a] + encodings[b] + encodings[c], "=");
    }

    return base64;
  }

  _configureAxios(environment) {
    const baseURL = environment === ENVIRONMENT.SANDBOX ? 'https://sandbox.boletobancario.com/boletofacil/integration/api' : 'https://www.boletobancario.com/boletofacil/integration/api';

    const instance = _axios.default.create({
      baseURL,
      headers: {
        'Content-Type': 'application/x-www-form-urlencoded'
      }
    });

    instance.interceptors.response.use(({
      data
    }) => data);
    return instance;
  }

}

var _default = JunoCardHash;
exports.default = _default;
//# sourceMappingURL=JunoCardHash.js.map